classdef Stage < dabs.interfaces.LinearStageController
    
    %% ABSTRACT PROPERTY REALIZATION (dabs.interfaces.LinearStageController)
    properties (Constant,Hidden)
        nonblockingMoveCompletedDetectionStrategy = 'poll'; % Either 'callback' or 'poll'
    end

    properties (SetAccess=protected,Dependent)
        isMoving;
    end

    properties (SetAccess=protected,Dependent,Hidden)
        invertCoordinatesRaw;
        positionAbsoluteRaw; 
        velocityRaw;
        accelerationRaw;
		maxVelocityRaw;        
    end
    
    properties (SetAccess=protected,Hidden)
        resolutionRaw = 1;
    end

    properties (SetAccess=protected)
        infoHardware = 'I am a XIMC stage!'; %String providing information about the hardware, e.g. firmware version, manufacture date, etc. Information provided is specific to each device type.
    end
    
    properties (SetAccess=protected,Hidden)
        positionDeviceUnits = 1.0e-6; %Units, in meters, in which the device's position values (as reported by positionAbsoluteRaw) are given
        velocityDeviceUnits = nan; %Units, in meters/sec, in which the device's velocity values (as reported by its hardware interface) are given. Value of NaN implies arbitrary units.
        accelerationDeviceUnits = nan; %Units, in meters/sec^2, in which the device's acceleration values (as reported by its hardware interface) are given. Value of NaN implies arbitrary units.         
    end
    
    %% DEVELOPER PROPERTIES
    properties (Hidden,SetAccess=protected)
        Position;
        device_id;
        calb;
    end
    
    %% CTOR/DTOR
    methods
        function obj = Stage(varargin)
            %% Init base class
            pvArgs = {'numDeviceDimensions' 1};
            obj = obj@dabs.interfaces.LinearStageController(pvArgs{:});
            
            %% Load ximc library
            if not(libisloaded('libximc'))
               disp('Load libximc library')
               [notfound,warnings] = loadlibrary('+dabs\+standa\libximc.dll', @dabs.standa.ximcm)
               disp('Load OK');
            end


            %% Get device name
            devArgs = most.util.filterPVArgs(varargin,{'device_name'});
            if isempty(devArgs)
                disp('ERROR: You must specify device_name in additional params');
                devArgs = {'device_name' 'xi-emu:\\\emu'};
            end
            device_name = devArgs{2};

            disp(['Using device name ', device_name]);

            %% Open device
            device_id = calllib('libximc','open_device', device_name);
            obj.device_id = device_id;

            disp(['Device id ', num2str(obj.device_id)]);
            
            %% Get user units
            unitArgs = most.util.filterPVArgs(varargin,{'meter_per_unit'});
            if isempty(unitArgs)
                disp('WARNING: You should specify meter_per_unit in additional params');
                disp('Default value is 1.0e-6');
                unitArgs = {'meter_per_unit' 1.0e-6};
            end
            
            obj.positionDeviceUnits = unitArgs{2};
            disp(['Meters per unit: ', num2str(obj.positionDeviceUnits)]);
            
            %% Create calibration struct
            calb = struct();
            calb.A = 1;
            calb.MicrostepMode = 9; % == MICROSTEP_MODE_FRAC_256
            %% Write calibration settings
            obj.calb = calb;


            %% Finish
            disp('XIMC stage initialized');
        end
        
        function delete(obj)
            if libisloaded('libximc')
                if obj.device_id > 0
                    disp('Close device');
                    device_id_ptr = libpointer('int32Ptr', obj.device_id);
                    calllib('libximc','close_device', device_id_ptr);
                end
            end
        end 

    end
    
    %% PROPERTY ACCESS
    methods
        function val = get.positionAbsoluteRaw(obj)
 
          dummy_struct = struct('Flags',999);
          parg_struct = libpointer('status_calb_t', dummy_struct);
          parg2_struct = libpointer('calibration_t', obj.calb);
          [result, res_struct] = calllib('libximc','get_status_calb', obj.device_id, parg_struct, parg2_struct);

          if result ~= 0
            disp(['Get position failed with code', num2str(result)]);
          end

          pos = res_struct.CurPosition;
          
		  val = pos;
        end
        
        function val = get.resolutionRaw(obj)
            val = 1;
        end
        
        function val = get.isMoving(obj)
            dummy_struct = struct('Flags', 0);
            parg_struct = libpointer('status_t', dummy_struct);
            [result, res_struct] = calllib('libximc','get_status', obj.device_id, parg_struct);
            clear parg_struct
            if result ~= 0
                disp(['Get status code', num2str(result)]);
                val = false;
            end

            if bitand(res_struct.MvCmdSts, 128) > 0
                val = true;
            else
                val = false;
            end
        end        
    end
    
    %% ABSTRACT METHOD IMPLEMENTATIONS
    methods (Access=protected,Hidden)

         function moveCompleteHook(obj,targetPosn)            
            disp(['Move to ...', num2str(targetPosn)]);
            
            calb_p = libpointer('calibration_t', obj.calb);
            result = calllib('libximc','command_move_calb', obj.device_id, targetPosn, calb_p);
            
            if result ~= 0
              disp(['Command move failed with error ', num2str(result)]);
            end

            calllib('libximc','command_wait_for_stop', obj.device_id, 100);
            
            obj.moveDone();

         end
        
        function moveStartHook(obj,targetPosn)
            disp(['Move to ...', num2str(targetPosn)]);
            
            calb_p = libpointer('calibration_t', obj.calb);
            result = calllib('libximc','command_move_calb', obj.device_id, targetPosn, calb_p);
            
            if result ~= 0
              disp(['Command move failed with error ', num2str(result)]);
            end
            
            obj.moveDone();
            
        end    
        
        function interruptMoveHook(obj)
            disp('Interrupt move hook not implemented');
            return;
        end           
        
        function recoverHook(obj)
            disp('Recover hook not implemented');
            return;
        end
        
        function resetHook(obj)
            disp('Reset hook not implemented');
            return;
        end
        
        function zeroHardHook(obj,coords)
            assert(all(coords),'Cannot hard-zero individual coordinates.');
            obj.Position = zeros(size(obj.Position));
        end    
    
    end    
end


%--------------------------------------------------------------------------%
% Stage.m                                                                  %
% Copyright  2016 Vidrio Technologies, LLC                                %
%                                                                          %
% ScanImage 2016 is premium software to be used under the purchased terms  %
% Code may be modified, but not redistributed without the permission       %
% of Vidrio Technologies, LLC                                              %
%--------------------------------------------------------------------------%
